'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
var mongo_bins_1 = require('./mongo-bins');
var Debug = require('debug');
var COMMAND = 'mongod';

// Taken from https://github.com/meteor/meteor/blob/debug-circle-timeout-promise-await/tools/utils/mongo-exit-codes.js
const MONGO_CODES = {
  0: {
    code: 0,
    symbol: 'EXIT_CLEAN',
    longText: 'MongoDB exited cleanly',
  },
  1: {
    code: 1,
    // No symbol in the source. This is in src/mongo/base/initializer.cpp.
    symbol: 'global-initialization',
    longText: 'MongoDB failed global initialization',
  },
  2: {
    code: 2,
    symbol: 'EXIT_BADOPTIONS',
    longText:
      'MongoDB was started with erroneous or incompatible command line options',
  },
  3: {
    code: 3,
    symbol: 'EXIT_REPLICATION_ERROR',
    longText:
      'There was an inconsistency between hostnames specified\n' +
      'on the command line compared with hostnames stored in local.sources',
  },
  4: {
    code: 4,
    symbol: 'EXIT_NEED_UPGRADE',
    longText: 'MongoDB needs to upgrade to use this database',
  },
  5: {
    code: 5,
    symbol: 'EXIT_SHARDING_ERROR',
    longText: 'A moveChunk operation failed',
  },
  12: {
    code: 12,
    symbol: 'EXIT_KILL',
    longText: 'The MongoDB process was killed, on Windows',
  },
  14: {
    code: 14,
    symbol: 'EXIT_ABRUPT',
    longText: 'Unspecified unrecoverable error. Exit was not clean',
  },
  20: {
    code: 20,
    symbol: 'EXIT_NTSERVICE_ERROR',
    longText: 'Error managing NT Service on Windows',
  },
  45: {
    code: 45,
    symbol: 'EXIT_FS',
    longText: 'MongoDB cannot open or obtain a lock on a file',
  },
  47: {
    code: 47,
    symbol: 'EXIT_CLOCK_SKEW',
    longText: 'MongoDB exited due to excess clock skew',
  },
  48: {
    code: 48,
    symbol: 'EXIT_NET_ERROR',
    longText:
      'MongoDB exited because its port was closed, or was already\n' +
      'taken by a previous instance of MongoDB',
  },
  100: {
    code: 100,
    symbol: 'EXIT_UNCAUGHT',
    longText:
      'MongoDB had an unspecified uncaught exception.\n' +
      'This can be caused by MongoDB being unable to write to a local database.\n' +
      `Check that you have permissions to write to your db path. MongoDB does\n` +
      'not support filesystems like NFS that do not allow file locking.',
  },
};

var MongodHelper = (function() {
  function MongodHelper(commandArguments) {
    if (commandArguments === void 0) {
      commandArguments = [];
    }
    this.resolveLink = function() {};
    this.rejectLink = function() {};
    this.mongoBin = new mongo_bins_1.MongoBins(COMMAND, commandArguments);
    this.debug = Debug('mongodb-prebuilt-MongodHelper');
  }
  MongodHelper.prototype.run = function() {
    var _this = this;
    return new Promise(function(resolve, reject) {
      _this.resolveLink = resolve;
      _this.rejectLink = reject;
      _this.mongoBin.run().then(function() {
        _this.mongoBin.childProcess.stderr.on('data', function(data) {
          return _this.stderrHandler(data);
        });
        _this.mongoBin.childProcess.stdout.on('data', function(data) {
          return _this.stdoutHandler(data);
        });
        _this.mongoBin.childProcess.on('close', function(code) {
          return _this.closeHandler(code);
        });
      });
    });
  };
  MongodHelper.prototype.closeHandler = function(code) {
    this.debug('mongod close: ' + code);
  };
  MongodHelper.prototype.stderrHandler = function(message) {
    this.debug('mongod stderr: ' + message);
  };
  MongodHelper.prototype.stdoutHandler = function(message) {
    this.debug('mongod stdout: ' + message);
    var log = message.toString();
    var mongodStartExpression = this.getMongodStartedExpression();
    var mongodAlreadyRunningExpression = this.getMongodAlreadyRunningExpression();
    var mongodPermissionDeniedExpression = this.getMongodPermissionDeniedExpression();
    var mongodDataDirNotFounddExpression = this.getMongodDataDirNotFounddExpression();
    var mongodShutdownMessageExpression = this.getMongodShutdownMessageExpression();
    if (mongodStartExpression.test(log)) {
      this.resolveLink(true);
    }
    if (mongodAlreadyRunningExpression.test(log)) {
      return this.rejectLink('already running');
    }
    if (mongodAlreadyRunningExpression.test(log)) {
      return this.rejectLink('already running');
    }
    if (mongodPermissionDeniedExpression.test(log)) {
      return this.rejectLink('permission denied');
    }
    if (mongodDataDirNotFounddExpression.test(log)) {
      return this.rejectLink('Data directory not found');
    }
    if (mongodShutdownMessageExpression.test(log)) {
      const trimmedLog = log.slice(log.indexOf('shutting down with error'), log.length);
      let error;
      if (trimmedLog) {
        Object.keys(MONGO_CODES).forEach((key) => {
          if (trimmedLog.indexOf(key) !== -1) {
            error = MONGO_CODES[key].longText;
          }
        });
      }
      return this.rejectLink(error ? error : 'Mongod shutting down with unspecified error.');
    }
  };
  MongodHelper.prototype.getMongodStartedExpression = function() {
    return /waiting for connections on port/i;
  };
  MongodHelper.prototype.getMongodAlreadyRunningExpression = function() {
    return /mongod instance already running/i;
  };
  MongodHelper.prototype.getMongodPermissionDeniedExpression = function() {
    return /permission denied/i;
  };
  MongodHelper.prototype.getMongodDataDirNotFounddExpression = function() {
    return /Data directory .*? not found/i;
  };
  MongodHelper.prototype.getMongodShutdownMessageExpression = function() {
    return /shutting down with code/i;
  };
  return MongodHelper;
})();
exports.MongodHelper = MongodHelper;
//# sourceMappingURL=/Users/winfinit/workspace/personal/mongodb-prebuilt/mongod-helper.js.map
